#------------------------------------------------------------------
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Header:
#
# $Author: kkorde $
# $Date: 2014/06/11 10:11:47 $
# $Id: OracleAS9.pm,v 1.11 2006/08/16 09:55:26 aprabhu1 Exp
# $Revision: 1.37 $
#
# Provides:                            OracleAS9
# Modification History:
# Known bugs/caveats:
#-------------------------------------------------------------------------------
# Be strict..
#-------------------------------------------------------------------------------
use strict;

#-------------------------------------------------------------------------------
# Declare the package..
#-------------------------------------------------------------------------------
package OracleAS9;

#-------------------------------------------------------------------------------
# Include required framework modules..
#-------------------------------------------------------------------------------
use IO::File;
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses
use Sys;            # support generalized UNIX commands
use Version;        # Version display generator
use Proc;           # For calls to the process routines

my ( $CLUS_HOME, $sPrefix, $bEnv, $bOnlineNode, @liRunningProcs ) = ();

#------------------------
# Setup VCS/VCSone home..
#------------------------
if ( $ENV { CLUSTER_HOME } ) { # VCS 5.x/VCSOne..
  $CLUS_HOME ||= $ENV { CLUSTER_HOME } ||= "/opt/VRTSvcsone";
}
else { # VCS 4.x..
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
}

( $sPrefix ) = $CLUS_HOME =~ /.*(vad|vcs(|one))$/;

my $sCmdHares  = $HARES  ||= "$CLUS_HOME/bin/hares";
my $sCmdHagrp  = $HAGRP  ||= "$CLUS_HOME/bin/hagrp";
my $sCmdHatype = $HATYPE ||= "$CLUS_HOME/bin/hatype";
my $sCmdHasys  = $HASYS  ||= "$CLUS_HOME/bin/hasys";

use constant EPOP                     =>  3;
use constant DONT_SOURCE_USER_ENV     =>  0;
use constant CHECK_INTERVAL           =>  5;
use constant MAXBUFFERSIZE            => 4096;

#-------------------------------------------------------------------------------
# Add our version number to the version hash and check if versions are
# requested.
#-------------------------------------------------------------------------------
Version::TellVersion("\$Id: OracleAS9.pm,v 1.24 2005/19/07 09:00:00 aprabhu 
Exp $::ETVHeader");


#-------------------------------------------------------------------------------
# Define OracleAS subroutines..
#-------------------------------------------------------------------------------

=pod

#-------------------------------------------------------------------------------
 Function :  bool ArgsValid ( IN String  EntryPointName,
                              IN String  User,
                              IN String  OracleHome,
                              IN String  ComponentProcessType,
                              IN integer SecondLevelMonitor,
                              IN String  MonitorProgram,
                              IN String  OracleInstance,
                              IN String  OracleInstanceHome); 

 Description : Validates the ArgList passed to the entry point.

 Parameters :

 $sEntryPointName               Entry point name
 $localUser                     UNIX user for Oracle Application Server
 $localOracleHome               ORACLE_HOME value for Oracle Application Server
 $localComponentProcessType     process-type associated with a ias component
 $localSecondLevelMonitor       Option to enable in-depth monitoring
 $localMonitorProgram           Name of the external monitor program
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
#-------------------------------------------------------------------------------

=cut

sub ArgsValid 
{
  #-----------------------------------------------------------------------------
  # Set up local vars from passed in ones..
  #-----------------------------------------------------------------------------
  my ( $sEntryPointName,
       $localUser,
       $localOracleHome,
       $localComponentProcessType,
       $localSecondLevelMonitor ,
       $localMonitorProgram,
       $localOracleInstance,
       $localOracleInstanceHome ) = @_;

  #-----------------------------------------------------------------------------
  # Announce Entry..
  #-----------------------------------------------------------------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName           [%s]
  User                     [%s]
  OracleHome               [%s]
  ComponentProcessType     [%s]
  SecondLevelMonitor       [%s]
  OracleInstanceName       [%s]
  OracleInstanceHome       [%s]",
 
  $sMyName,  $sEntryPointName,  $localUser,  $localOracleHome, 
  $localComponentProcessType, $localSecondLevelMonitor, $localOracleInstance,
  $localOracleInstanceHome );

  #-----------------------------------------------------------------------------
  # Set return code to hopeful success..
  #-----------------------------------------------------------------------------
  my $iReturnCode = 1;

  #-----------------------------------------------------------------------------
  # Create hash to weed out empty args values..
  #-----------------------------------------------------------------------------
  my ( @lsMissing, %hsArgList ) = ();
  my @lsArgs =
  qw ( EntryPointName User OracleHome ComponentProcessType SecondLevelMonitor );

  @hsArgList { @lsArgs } = @_; # Assign..

  #-----------------------------------------------------------------------------
  # Reqd. attributes based on entry point..
  #-----------------------------------------------------------------------------
  my @lsOnlineArgs  = qw ( User OracleHome ComponentProcessType );
  my @lsOfflineArgs = qw ( User OracleHome ComponentProcessType );
  my @lsMonitorArgs = qw ( User OracleHome ComponentProcessType );
  my @lsCleanArgs   = qw ( User OracleHome ComponentProcessType );
  my @lsImfRegisterArgs = qw ( User OracleHome ComponentProcessType );

  my %hlsNeeded = ( 'online'  => \@lsOnlineArgs,
                    'offline' => \@lsOfflineArgs,
                    'monitor' => \@lsMonitorArgs,
                    'clean'   => \@lsCleanArgs,
                    'imf_register'	=> \@lsImfRegisterArgs );

  my @lsReqdArgs = @{$hlsNeeded { $sEntryPointName }};

  #-----------------------------------------------------------------------------
  # Check what's missing..
  #-----------------------------------------------------------------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20002,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = 0; 
  }

  #-----------------------------------------------------------------------------
  # Agent specific validation starts 
  #-----------------------------------------------------------------------------

  if ( $sEntryPointName eq "online"  || 
       $sEntryPointName eq "offline" )
  {
    if ( ! -d $localOracleHome )
    {
      &logError(20003,"<%s>:OracleHome [%s] is not accessible.",
        $sMyName,$localOracleHome);
      $iReturnCode = 0 ;
    }
    if ( ($localOracleInstanceHome =~ /\S+/) && (! -d $localOracleInstanceHome) ) 
    {
       &logError(20310,"<%s>:OracleInstanceHome [%s] is not accessible.",
                 $sMyName, $localOracleInstanceHome);
       $iReturnCode = 0 ;
    }   
  }

  my $spwname = getpwnam($localUser);
  if ( ! defined $spwname )
  {
    &logError(20004, "<%s>:User name [%s] invalid.",$sMyName,$localUser);
    $iReturnCode = 0 ;
  }

  #-----------------------------------------------------------------------------
  # Validate MonitorProgram if passed during online
  #-----------------------------------------------------------------------------
  if ( $sEntryPointName eq "online" )
  {
    if ( $localMonitorProgram )
    {
      if ( ! -x $localMonitorProgram )
      {
        &logError(20005,"<%s> MonitorProgram [%s] not executable. 
                              Check the full pathname and the permissions",
                              $sMyName, $localMonitorProgram);
        $iReturnCode = 0 ;
      }
    }
  }

  $iReturnCode || return $iReturnCode;

  $iReturnCode =  OracleAS9::ValidateComponent ( $localOracleHome, 
                                                 $localComponentProcessType,
                                                 $sEntryPointName,
                                                 $localOracleInstance,
                                                 $localOracleInstanceHome 
                                               );
 
  if ( 0 == $iReturnCode )
  {
    #---------------------------------------------------------------------------
    # Some arguments validation failed.
    #---------------------------------------------------------------------------
    &logError ( 20006,"<%s> Some of the attributes validation failed",$sMyName);
  }

  return $iReturnCode;
 
}

=pod

#-------------------------------------------------------------------------------
 Function :  bool ValidateComponent ( IN String  OracleHome,
                                      IN String  ComponentProcessType,
                                      IN String  EntryPointName );

 Description : Validates that the component defined by the user is a valid 
               componentt

 Parameters :

 $localOracleHome               ORACLE_HOME value for Oracle Application Server
 $localComponentProcessType     Name of the specific Oracle AS component
 $localEntryPointName            Entry Point Name
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
#-------------------------------------------------------------------------------

=cut

sub ValidateComponent 
{
  #-----------------------------------------------------------------------------
  # Set up local vars from passed in ones..
  #-----------------------------------------------------------------------------
  my ( $localOracleHome,
       $localComponentProcessType,
       $localEntryPointName,
       $localOracleInstance,
       $localOracleInstanceHome ) = @_;

  #-----------------------------------------------------------------------------
  # Announce Entry..
  #-----------------------------------------------------------------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20007, "Subroutine <%s> called with:
  OracleHome               [%s]
  ComponentProcessType     [%s]
  OracleInstance           [%s]
  OracleInstanceHome       [%s]",
  $sMyName,  $localOracleHome, $localComponentProcessType, $localOracleInstance, $localOracleInstanceHome );

  #-----------------------------------------------------------------------------
  # ias-component    process-type
  # ---------------------------------
  # HTTP_Server	     HTTP_Server
  # OID		     OID
  # OVD              OVD
  # WebCache 	     WebCache, WebCacheAdmin
  # Discoverer	     OAD, OSAgent, ServicesStatus, PreferenceServer, SessionServer
  # OC4J	     home, OC4J_SECURITY, OC4J_Portal, OC4J_BI_Forms
  # wireless	     OC4J_Wireless, performance_server, messaging_server	
  #-----------------------------------------------------------------------------
  my @compProc = qw( OPMN HTTP_Server OID WebCache WebCacheAdmin
                     OAD OSAgent ServicesStatus PreferenceServer SessionServer
                     home OC4J_SECURITY OC4J_Portal OC4J_BI_Forms 
                     OC4J_Wireless performance_server messaging_server 
                     dcm-daemon iasconsole dbconsole agent OHS);

  if( $localOracleInstance =~ /\S+/ && $localOracleInstanceHome =~ /\S+/ ){
      @compProc = qw( OPMN OID OVD OHS ReportsServerComponent WebCache WebCacheAdmin ServicesStatus PreferenceServer agent );
  }

  my %param = map { $_ => 1 } @compProc;
  if( (! exists ( $param{$localComponentProcessType} ) ) &&
      ( $localOracleInstance =~ /\S+/ && $localOracleInstanceHome =~ /\S+/ )){
      &logError(20314, "<%s>:Component Process Type [%s] not supported by the agent for Oracle Fusion Middleware instance [%s].", $sMyName, $localComponentProcessType, $localOracleInstance);    
      return 0;
  } elsif ( exists ( $param{$localComponentProcessType} ) ) {
      return 1;
  }
  
  #-----------------------------------------------------------------------------
  # We are here because the component is not in our list. Could be a custom
  # j2ee instance.  So check under $OracleHome/j2ee but only for online/offline
  # during monitor the dir will not exist on the offline node
  #-----------------------------------------------------------------------------
  if ( $localEntryPointName eq "online" || $localEntryPointName eq "offline" )
  {
    my $j2eeDirName = $localOracleHome."/j2ee/".$localComponentProcessType ;
    return 1 if ( -e  $j2eeDirName ) ;

    &logError(20315, "<%s>:Component Process Type [%s] not supported by the agent.", $sMyName, $localComponentProcessType);
    return 0;
  }
  return 1;

}

=pod


#-------------------------------------------------------------------------------
 Function : bool Start ( IN String  User,
                         IN String  OracleHome,
                         IN String  ComponentProcessType,
                         IN integer SecondLevelMonitor,
                         IN String  ResourceType,
                         IN String  ResourceName );

 Description : Execute the start command needed to start the Oracle AS component

 Parameters :

 $sUser                    UNIX user for Oracle Application Server
 $sOracleHome              ORACLE_HOME value for Oracle Application Server
 $sComponentProcessType    Component Name of the specific OracleAS component
 $iSecondLevelMonitor      Specifies how frequently the SecondLevel Monitor runs
 $sResourceType            The Resource Type of this component 
 $sResourceName            The Resource Name of this component 
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
#-------------------------------------------------------------------------------

=cut

sub Start 
{
  #-----------------------------------------------------------------------------
  # Get Arguments..
  #-----------------------------------------------------------------------------
  my ( $sUser,
       $sOracleHome,
       $sOracleInstance,
       $sOracleInstanceHome,
       $sComponentProcessType,
       $iSecondLevelMonitor,
       $sResourceType,
       $sResourceName ) = @_;

  #-----------------------------------------------------------------------------
  # Announce Entry..
  #-----------------------------------------------------------------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20008, "Subroutine <%s> called with: 
  User                     [%s]
  OracleHome               [%s]
  OracleInstanceName       [%s]
  OracleInstanceHome       [%s]
  ComponentProcessType     [%s]
  SecondLevelMonitor       [%s] 
  ResourceType             [%s] 
  ResourceName             [%s] ",

  $sMyName, $sUser, $sOracleHome, $sOracleInstance, $sOracleInstanceHome, $sComponentProcessType, $iSecondLevelMonitor, 
  $sResourceType, $sResourceName );

  #-----------------------------------------------------------------------------
  # Check the Install type - Infrastructure or Application Server
  #-----------------------------------------------------------------------------
  
  my $sInstallType = OracleAS9::GetInstallType($sOracleHome);

  &logTrace(20009, "<%s> Starting Component [%s] of AS  type  [%s] ",
                    $sMyName, $sComponentProcessType, $sInstallType);

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #--------------------------------------
  # Remove lock file, if any            
  #--------------------------------------
 
  my ( $sLockFilesDir, @lsFiles, $sLockFile ) =  0;

  # Collect "emctl.pid" for "iasconsole" and "dbconsole" ProcessType
  if ( $sComponentProcessType eq "iasconsole" || $sComponentProcessType eq "dbconsole" )
  {
     $sLockFilesDir = "$sOracleHome";
     $sLockFile = "emctl.pid";
     if ( -e  "$sLockFilesDir/$sLockFile" ){
        push ( @lsFiles, "$sLockFilesDir/$sLockFile" );
     }
     elsif ( -e "$sLockFilesDir/bin/$sLockFile" ) {
        push ( @lsFiles, "$sLockFilesDir/bin/$sLockFile" );
     }
  }

  # Collect "httpd.pid" for "HTTP_Server" ProcessType 
  if (  $sComponentProcessType eq "HTTP_Server" )
  {
     $sLockFilesDir = "$sOracleHome/Apache/Apache/logs";
     $sLockFile = "httpd.pid";
     if ( -e  "$sLockFilesDir/$sLockFile" ){
        push ( @lsFiles, "$sLockFilesDir/$sLockFile" );
     }
  }

  # Collect "jms.state" for OC4J componant ProcessTypes
  $sLockFilesDir = "$sOracleHome/j2ee/" . $sComponentProcessType . "/persistence/" . $sComponentProcessType . "_default_group_1/";
  $sLockFile      = ( "jms.state" ); 
  if ( -e  "$sLockFilesDir/$sLockFile" ){
     push ( @lsFiles, "$sLockFilesDir/$sLockFile" );
  }

  # Collect all state lock files managed by "opmn"
  if ( $sComponentProcessType ne "agent" && $sComponentProcessType ne "dbconsole" && $sComponentProcessType ne "iasconsole" )
  {
     $sLockFilesDir =  "$sOracleHome/opmn/logs/states";
     #opendir(DIR, $sLockFilesDir) or die "can't opendir $sLockFilesDir: $!";
     if ( opendir(DIR, $sLockFilesDir) ) {
         while (defined($sLockFile = readdir(DIR))) {
            next if $sLockFile =~ /^\./;     # skip files starting with .
            &logTrace(20011,"<%s> Found lock file : <%s> ", $sMyName, $sLockFile);
            push ( @lsFiles, "$sLockFilesDir/$sLockFile" );
         }
         closedir(DIR);
     }
     else {
         &logTrace(20232,"<%s>:Can't opendir [%s]", $sMyName,$sLockFilesDir );
     }
  }

  # Print list of files to be removed before ONLINE
  &logTrace(20099,"<%s>:List of Files to remove [%s] ", $sMyName,
                   join (',',@lsFiles) ); 

  #----------------------
  # Removing Lock files
  #----------------------
  if ( scalar @lsFiles && ! Sys::UnlinkFilesAsUser ( @lsFiles ) )
  {
      &logError ( 20012, "<%s> Failed to remove stale file(s) [%s]",
                 $sMyName, join ( ' ', @lsFiles ) );
      $iReturnCode = 0; # Failure..
  }  

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + OracleAS9::GetEngineTimeout ( 1 );

  #-----------------------------------------------------------------------------
  # Start Oracle Application Server processes by running the start command
  #-----------------------------------------------------------------------------
  
  my $sEnvFile       = "";
  my $iSourceEnvFlag = 0;
  my $iExitCode      = 0;
  my @lsCommandOut   = ();
  my $sStartScript   = "";
  my $sStartOptions  = "";
  my $j2eeDirName    = $sOracleHome."/j2ee/".$sComponentProcessType ;
  my $bTimedOut = 0;
  my $iTimeOut;

  if ( $sOracleInstance =~ /\S+/ && $sOracleInstanceHome =~ /\S+/ ){
       $sStartScript = "$sOracleInstanceHome/bin/opmnctl";
  } else {
       $sStartScript = "$sOracleHome/opmn/bin/opmnctl";
  }

  if ( $sComponentProcessType eq "OPMN" ){
       $sStartOptions = "start";
  } elsif ( $sComponentProcessType eq "iasconsole" || $sComponentProcessType eq "dbconsole" || 
            $sComponentProcessType eq "agent" ){
       if ( $sOracleInstance =~ /\S+/ &&  $sOracleInstanceHome =~ /\S+/ ){
            $sStartScript = "$sOracleInstanceHome/EMAGENT/emagent_$sOracleInstance/bin/emctl";
       } else {     
            $sStartScript = "$sOracleHome/bin/emctl";
       }
       $sStartOptions = "start $sComponentProcessType";
  } elsif ( $sComponentProcessType eq "dcm-daemon" ) {
       $sStartOptions = "startproc ias-component=$sComponentProcessType";
  } else {  # Default is process-type based startup
       $sStartOptions = "startproc process-type=$sComponentProcessType";
       if ( $sOracleInstance =~ /\S+/ ){
            if($sComponentProcessType eq "WebCacheAdmin"){
               # The opmnctl status displays "WebCacheAdmin" component process type as "WebCache-admin" for Oracle Fusion
               # Middleware installations
               $sStartOptions = "startproc process-type=WebCache-admin";
            } elsif ( $sComponentProcessType eq "OID" ){
               # The opmnctl status displays "OID" Component Process type as "oidmon" for Oracle Fusion Middleware installations
               $sStartOptions = "startproc process-type=oidmon";
            }
       } 
  }
  #------------------------------------
  # Check if StartScript is executable
  #------------------------------------
  if (! -x $sStartScript )
  {
    &logError(20013,"<%s>:Start script [%s] is not executable. 
                    Check if it exists and is executable ",
                    $sMyName,$sStartScript);
    return 0;
  }
  &logTrace(20014,"<%s>:Running command [%s] with [%s] ", $sMyName,  
                   $sStartScript , $sStartOptions );
  #------------------
  # Request startup..
  #------------------
  if ( ( $iTimeOut = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20233,
    "<%s> Insufficient time to continue boot up. Short by [%s] sec",
    $sMyName, abs ( $iTimeOut ) );

    $iReturnCode = 0; # Failure..
  }
  else { 
    $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                  $iSourceEnvFlag,
                                                  $sStartScript,
                                                  $sStartOptions,
                                                  \$iExitCode,
                                                  $sUser,
                                                  \@lsCommandOut,
                                                  \$bTimedOut,
                                                  $iTimeOut,
                                                  "",
                                                  "");
  }

  if ( $iReturnCode )
  {
    #---------------------------------------------------------------------------
    # Command got executed.
    #---------------------------------------------------------------------------
    if ( $iExitCode )
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20016,"<%s>:[%s] with args [%s] returned exit code [%s]",
                       $sMyName,$sStartScript,$sStartOptions,$iExitCode);
      return 0;
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command executed successfully.
      #-------------------------------------------------------------------------
      &logTrace (20017, "<%s>:[%s] executed successfully.",
                         $sMyName, $sStartScript);
      return 1;
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #-------------------------------------------------------------------------
      # Command timed out.
      #-------------------------------------------------------------------------
      &logError (20018,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sStartScript,$iTimeOut);
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20019,"<%s>:[%s] failed.", $sMyName,$sStartScript);
    }
    return 0;
  }
}

=pod

#-------------------------------------------------------------------------------
 Function : bool Stop ( IN String User,
                        IN String OracleHome,
                        IN String ComponentProcessType,
                        IN String ResourceType,
                        IN String ResourceName );

 Description : Execute the stop command needed to start the Oracle AS component

 Parameters :

 $sUser                    UNIX user for Oracle Application Server
 $sOracleHome              ORACLE_HOME value for Oracle Application Server
 $sComponentProcessType           Component Name of the specific OracleAS component
 $sResourceType            The Resource Type of this component
 $sResourceName            The Resource Name of this component
 

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
#-------------------------------------------------------------------------------

=cut

sub Stop 
{
  #-----------------------------------------------------------------------------
  # Get Arguments..
  #-----------------------------------------------------------------------------
  my ( $sUser,
       $sOracleHome,
       $sOracleInstance,
       $sOracleInstanceHome,
       $sComponentProcessType,
       $sResourceType,
       $sResourceName ) = @_ ;

  #-----------------------------------------------------------------------------
  # Announce Entry..
  #-----------------------------------------------------------------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20020, "Subroutine <%s> called with:
  User                     [%s]
  OracleHome               [%s]
  OracleInstanceName       [%s]
  OracleInstanceHome       [%s]
  ComponentProcessType     [%s]
  ResourceType             [%s]
  ResourceName             [%s] ",

  $sMyName, $sUser, $sOracleHome, $sOracleInstance, $sOracleInstanceHome, $sComponentProcessType, $sResourceType, $sResourceName);

  #-----------------------------------------------------------------------------
  # Check the Install type - Infrastructure or Application Server
  #-----------------------------------------------------------------------------

  my $sInstallType = OracleAS9::GetInstallType($sOracleHome);
  &logTrace(20021, "<%s> Stopping  Component [%s] of OracleAS  type = [%s] ", 
                    $sMyName, $sComponentProcessType, $sInstallType);

  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + OracleAS9::GetEngineTimeout ( 1 );

  #-----------------------------------------------------------------------------
  # Set Return Code Optimistically to 1..
  #-----------------------------------------------------------------------------
  my $iReturnCode = 1;

  #-----------------------------------------------------------------------------
  # Remove the SLM file
  #-----------------------------------------------------------------------------
  &logTrace(20023,"<%s>: Unlinking [%s]",$sMyName,
                  "/tmp/.VRTS$sResourceType/$sResourceName.slm");
  unlink ("/tmp/.VRTS$sResourceType/$sResourceName.slm");

  #-----------------------------------------------------------------------------
  # Stop Oracle Application Server processes by running the stop command
  #-----------------------------------------------------------------------------
  my $sEnvFile        = "";
  my $iSourceEnvFlag  = 0;
  my $iExitCode       = 0;
  my @lsCommandOut    = ();
  my $sStopScript     = "";
  my $sStopOptions    = "";
  my $bTimedOut = 0;
  my $iTimeOut;
  my $j2eeDirName = $sOracleHome."/j2ee/".$sComponentProcessType ;

  if ( $sOracleInstance =~ /\S+/ && $sOracleInstanceHome =~ /\S+/ ){
      $sStopScript  = "$sOracleInstanceHome/bin/opmnctl";
  } else {   
      $sStopScript  = "$sOracleHome/opmn/bin/opmnctl";
  }

  if ( $sComponentProcessType eq "OPMN" )
  {
     $sStopOptions = "stopall";  # There is no "opmnctl stop" command
  } elsif ( $sComponentProcessType eq "iasconsole" || 
            $sComponentProcessType eq "dbconsole" ||
            $sComponentProcessType eq "agent" ) {
     if( $sOracleInstance =~ /\S+/ && $sOracleInstanceHome =~ /\S+/ ){
        $sStopScript   = "$sOracleInstanceHome/EMAGENT/emagent_$sOracleInstance/bin/emctl";
     } else {
        $sStopScript   = "$sOracleHome/bin/emctl";
     }
     $sStopOptions  = "stop $sComponentProcessType";
  } elsif ( $sComponentProcessType eq "dcm-daemon" ) {
     $sStopOptions = "stopproc ias-component=$sComponentProcessType";
  } else {
     # Default is process-type based stop
     $sStopOptions = "stopproc process-type=$sComponentProcessType";
     if( $sOracleInstance =~ /\S+/ ) {
         # The opmnctl status displays "WebCacheAdmin" Component Process type as "WebCache-admin" for Oracle Fusion
         # Middleware installations
         if ( $sComponentProcessType eq "WebCacheAdmin" ){
              $sStopOptions = "stopproc process-type=WebCache-admin";
         } elsif ( $sComponentProcessType eq "OID" ){
              # The opmnctl status displays "OID" Component Process type as "oidmon" for Oracle Fusion Middleware installations
              $sStopOptions = "stopproc process-type=oidmon";
         }
     }
  }
  #-----------------------------------
  # Check if StopScript is executable
  #-----------------------------------
  if (! -x $sStopScript )
  {
    &logError(20234,"<%s>:Stop script [%s] is not executable.
                    Check if it exists and is executable ",
                    $sMyName,$sStopScript);
    return 0;
  }
  &logTrace(20015,"<%s>:Running command [%s] with [%s] ", $sMyName,
                   $sStopScript , $sStopOptions );

  #------------------------
  # Request shutdown..
  #------------------------
  if ( ( $iTimeOut = $iTillWhen - time ) <= 10 ) { # Get time bearings..
    &logError ( 20235,
    "<%s> Insufficient time to continue shutdown. Short by [%s] sec",
    $sMyName, abs ( $iTimeOut ) );

    $iReturnCode = 0; # Failure..
  }
  else {
    $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                  $iSourceEnvFlag,
                                                  $sStopScript,
                                                  $sStopOptions,
                                                  \$iExitCode,
                                                  $sUser,
                                                  \@lsCommandOut,
                                                  \$bTimedOut,
                                                  $iTimeOut,
                                                  "",
                                                  "");
  }

  if ( $iReturnCode )
  {
    #---------------------------------------------------------------------------
    # Command got executed.
    #---------------------------------------------------------------------------
    if ( $iExitCode )
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20024,"<%s>:[%s] with args [%s] returned exit code [%s]",
                 $sMyName,$sStopScript,$sStopOptions,$iExitCode);
      return 0;
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command executed successfully.
      #-------------------------------------------------------------------------
      &logTrace (20025, "<%s>:[%s] executed successfully.",
                 $sMyName, $sStopScript);
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #-------------------------------------------------------------------------
      # Command timed out.
      #-------------------------------------------------------------------------
      &logError (20026,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sStopScript,$iTimeOut);
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20027,"<%s>:[%s] failed.", $sMyName,$sStopScript);
    }
    return 0;
  }

  #------------------------------------------------------------------
  # Implement DelayEntryPoint till OfflineTimeout value if the
  # Server appears to be running even after the script executed Successfully 
  #------------------------------------------------------------------
 
  if ( $iReturnCode )
  {
     if ( DelayEntryPoint ( $iTillWhen,
                            $sUser,
                            $sOracleHome, 
                            $sOracleInstance,
                            $sOracleInstanceHome,
                            $sComponentProcessType ) )
     {
        &logTrace( 20028, "<%s>:Successfully delayed the entry point.",$sMyName);
        $iReturnCode = 1;
     }
     else
     {
        &logError( 20029, "<%s>:Server process running even after DelayEntryPoint.",$sMyName);
        $iReturnCode = 0;
     }           
  }
  return $iReturnCode;
}

=pod

#--------------------------------------------------------------------------
 Function : bool Kill (  IN String User,
                         IN String OracleHome,
                         IN String ComponentProcessType );

 Description : Cleanup processes after a failed Offline

 Parameters :

 $sUser                    UNIX user for Oracle Application Server
 $sOracleHome              ORACLE_HOME value for Oracle Application Server
 $sComponentProcessType           Component Name of the specific OracleAS component
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
#-------------------------------------------------------------------------

=cut

sub Kill 
{
  #----------------
  # Get Arguments..
  #----------------
  my ( $sUser,
       $sOracleHome ,
       $sOracleInstance, 
       $sOracleInstanceHome, 
       $sComponentProcessType ) = @_;

  #-----------------------------------------------------------------------------
  # Announce Entry..
  #-----------------------------------------------------------------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20030, "Subroutine <%s> called with:
  User                     [%s]
  OracleHome               [%s]
  OracleInstanceName       [%s]
  OracleInstanceHome       [%s]
  ComponentProcessType     [%s]",
  $sMyName, $sUser, $sOracleHome, $sOracleInstance, $sOracleInstanceHome, $sComponentProcessType );

  #-----------------------------------------------------------------------------
  # Set Return Code Optimistically to 1..
  #-----------------------------------------------------------------------------
  my $iReturnCode = 1;

  #-----------------------------------------------------------------------------
  # Identify and kill processess
  #-----------------------------------------------------------------------------
  my @rhlProcsMatched = ();
  my @liCompPids = ();

  $iReturnCode = GetProcState ($sUser,
                               $sOracleHome,
                               $sOracleInstance, 
                               $sOracleInstanceHome, 
                               $sComponentProcessType,
                               \@rhlProcsMatched );

  if ( VCSDef_MonitorExit::OFFLINE != $iReturnCode )
  {
    my @liCompPids = map { $_ -> { PID } } @rhlProcsMatched;
    if (Proc::KillPidsFromList(\@liCompPids))
    {
      &logTrace(20031,"<%s> Killed OAS Component [%s] processes.",
                       $sMyName, $sComponentProcessType);
    }
    else
    {
      &logError(20032,"<%s>:Error killing OAS script processes.",$sMyName);
      $iReturnCode = 0;

    }
  }

  return $iReturnCode;
}

=pod

#------------------------------------------------------------------------------
 Function : int GetResourceState ( IN String  User,
                                   IN String  OracleHome,
                                   IN String  ComponentProcessType,
                                   IN String  MonitorProgram,
                                   IN integer SecondLevelMonitor,
                                   IN String  ResourceType,
                                   IN String  ResourceName );

 Description : Determine the start of the resource

 Parameters :

 $sUser                    UNIX user for Oracle Application Server
 $sOracleHome              ORACLE_HOME value for Oracle Application Server
 $sComponentProcessType    Component Name of the specific OracleAS component
 $sMonitorProgram          The external monitor program specified by user
 $iSecondLevelMonitor      Specifies how frequently the SecondLevel Monitor runs
 $sResourceType            The Resource Type of this component
 $sResourceName            The Resource Name of this component

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
#-------------------------------------------------------------------------------

=cut

sub GetResourceState 
{
  #-----------------------------------------------------------------------------
  # Get Arguments..
  #-----------------------------------------------------------------------------
  my ( $sUser ,
       $sOracleHome, 
       $sComponentProcessType,
       $sMonitorProgram, 
       $iSecondLevelMonitor, 
       $sResourceType, 
       $sResourceName,
       $level_one,
       $level_two,
       $sOracleInstance,
       $sOracleInstanceHome ) = @_;

  my $iReturnCode = 1;
  #-----------------------------------------------------------------------------
  # Announce Entry..
  #-----------------------------------------------------------------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20033, "Subroutine <%s> called with 
  User                     [%s]
  OracleHome               [%s]
  OracleInstanceName       [%s]
  OracleInstanceHome       [%s]
  ComponentProcessType     [%s]
  MonitorProgram           [%s] 
  SecondLevelMonitor       [%s]
  ResourceType             [%s]
  ResourceName             [%s]
  level_one                [%s]
  level_two                [%s]",
  $sMyName,$sUser, $sOracleHome, $sOracleInstance, $sOracleInstanceHome, $sComponentProcessType, $sMonitorProgram,
  $iSecondLevelMonitor, $sResourceType, $sResourceName, $level_one, $level_two );
   
  #---------------------------------------------------------------------
  # Upper bound.. Time in EPOCH when entry pt started + VCS time limit..
  #---------------------------------------------------------------------
  my $iTillWhen = $^T + OracleAS9::GetEngineTimeout ( 1 );
  my $iTimeOut = $iTillWhen - time; 

  if ( 1 == $level_one ) {
    #-----------------------------------------------------------------------------
    # Do a first level ( process level check )
    #-----------------------------------------------------------------------------
    $iReturnCode = OracleAS9::GetProcState( $sUser ,
                                            $sOracleHome,
                                            $sOracleInstance,
                                            $sOracleInstanceHome,
                                            $sComponentProcessType);
  
    if ( VCSDef_MonitorExit::ONLINE == $iReturnCode )
    {
      &logTrace(20034,
      "<%s> GetResourceState found all required processes", $sMyName);
    }
    else  ## VCSDef_MonitorExit:OFFLINE
    {
      &logTrace(20035,
      "<%s> GetResourceState found that resource was offline", $sMyName);
      return $iReturnCode;
    }
  }
  
  #---------------------------------------
  # Do Second level monitoring if enabled.
  #---------------------------------------
  if ( -300 == $level_two ) {
     $level_two = RunSecondLevelMonitor ( $iSecondLevelMonitor );  
     &logTrace ( 20307, "<%s> Traditional second level monitoring, SLM flag for this monitor set to [%s] ", $sMyName, $level_two  );
  }

  if ( $level_two >= 1 )
  {
  #-----------------------------------------------------------------------------
  # We reached here because GetProcState returned  the resource to be online.
  # Call SecondLevelMonitor
  # Execute External MonitorProgram if specified.
  #-----------------------------------------------------------------------------

    #-------------------------------------------------------------------------
    # Call SecondLevelMonitor
    #-------------------------------------------------------------------------

    $iReturnCode = SecondLevelMonitor($sOracleHome,
                                      $sUser,
                                      $sComponentProcessType,
                                      $iTimeOut,
                                      $sOracleInstance,
                                      $sOracleInstanceHome);

    if (   VCSDef_MonitorExit::UNKNOWN == $iReturnCode
        || VCSDef_MonitorExit::OFFLINE == $iReturnCode )
    {
       return $iReturnCode;
    }
    elsif( $iReturnCode == VCSDef_MonitorExit::ONLINE )
    {
      #--------------------------------------------------------------
      # Fall through to check if there is a MonitorProgram defined.
      #--------------------------------------------------------------
    }
    else
    {
      &logError( 20094, "<%s>:Second level monitor procedures for the OracleAS9 resource failed to run or timedout", $sMyName );
      return VCSDef_MonitorExit::UNKNOWN;
    }  
  }

  #---------------------------------------
  # if MonitorProgram is set , execute it.
  #---------------------------------------
  if ( $sMonitorProgram )
  {
    my $sEnvFile = "";
    my $iSourceEnvFlag = 0;
    my $iExitCode;
    my @lsCommandOut;
    my $bTimedOut = 0;

    $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                    $iSourceEnvFlag,
                                                    $sMonitorProgram,
                                                    "",
                                                    \$iExitCode,
                                                    $sUser,
                                                    \@lsCommandOut,
                                                    \$bTimedOut,
                                                    $iTimeOut,
                                                    "",
                                                    "");
    if ( $iReturnCode )
    {
      #----------------------
      # Command got executed.
      #----------------------
      if ( 100 == $iExitCode || 1 == $iExitCode )
      {
        #------------------------------------------------------
        # Monitor Program returned OAS instance to be offline.
        #------------------------------------------------------
        &logTrace (20041,"<%s>:[%s] returned exit code [%s]",
                   $sMyName,$sMonitorProgram,$iExitCode);
        &logError (20042,"<%s>:[%s] reports resource state PARTIAL_ONLINE",
                   $sMyName,$sMonitorProgram);
        return VCSDef_MonitorExit::PARTIAL_ONLINE;
      }
      elsif ( 110 == $iExitCode || 0 == $iExitCode )
      {
        #------------------------------------------------------
        # Monitor Program returned OAS instance to be online.
        #------------------------------------------------------
        &logTrace (20043, "<%s>: [%s] return exit code [%d]",
                  $sMyName, $sMonitorProgram, $iExitCode);
        return VCSDef_MonitorExit::ONLINE;
      }
      else
      {
        #-------------------------
        # OAS status is Unknown
        #-------------------------
        &logError (20044,"<%s>:[%s] reports resource state UNKNOWN",
                   $sMyName,$sMonitorProgram);
        return VCSDef_MonitorExit::UNKNOWN;
      }
    }
    else
    {
      if ( $bTimedOut )
      {
        #-------------------
        # Command timed out.
        #-------------------
        &logError (20045,"<%s>:[%s] did not complete within [%s] seconds.",
                   $sMyName,$sMonitorProgram,$iTimeOut);
      }
      else
      {
        #-------------------
        # Command failed.
        #-------------------
       &logError (20046,"<%s>:[%s] failed.",
                   $sMyName,$sMonitorProgram);
      }
      #-------------------------
      # OAS status is Unknown
      #-------------------------
      return VCSDef_MonitorExit::UNKNOWN;
    }
  }
  #-----------------------------------------------------------------------------
  # If we are here, nothing declared the component as offline or unknown.
  #-----------------------------------------------------------------------------
  return VCSDef_MonitorExit::ONLINE;
}

=pod

 -------------------------------------------------------------------------
 Function : int GetProcState ( IN String  User,
                               IN String  OracleHome,
                               IN String  ComponentProcessType,
                               OUT ListOfProcs ProcsMatched );

 Description : Determine if for a given Component, if any of its processes
               are running
 Parameters :
 $sUser                    UNIX user for Oracle Application Server
 $sOracleHome              ORACLE_HOME value for Oracle Application Server
 $sComponentProcessType    Component Name of the specific OracleAS component
 $rhlProcsMatched          List of Matched Procs 

 Returns : Integer value as defined below:
           0  No matching processes found or error getting the processes
           1  Matching processes found or error getting the processes

 -------------------------------------------------------------------------

=cut

sub GetProcState
{
  #------------------
  # Get Arguments
  #------------------
  my ( $sUser,
       $sOracleHome,
       $sOracleInstance,
       $sOracleInstanceHome,
       $sComponentProcessType,
       $rhlProcsMatched ) = @_ ;

  #------------------
  # Announce Entry
  #------------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20047, "Subroutine <%s> called:
                      User       [%s]
                      OracleHome [%s]
                      OracleInstanceName [%s]
                      OracleInstanceHome [%s]
                      ComponentProcessType  [%s]",
                      $sMyName, $sUser, $sOracleHome, $sOracleInstance, $sOracleInstanceHome, $sComponentProcessType);

  #---------------------------------------------
  # Set Return Code Optimistically to ONLINE
  #---------------------------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;

  my $iZsched1;
  my $iZsched2;
  my $bZoneFlag = 0;
  my $sOSType = Arch::GetOSType();

  @$rhlProcsMatched = (); # Truncate  ..

  #-----------------------------------
  # Call processes and store in hash
  #-----------------------------------
  if ($sOracleInstance =~ /\S+/){
      $bEnv = 1;
  }
  my @hProcListHash = (); #set this empty;
  my $iProcessCount = Proc::GetLongProcessListHash(\@hProcListHash, $bEnv);

  my @hUidProcListHash = ();
  my $iUid = getpwnam $sUser;
  my $iMatchedPids = Proc::FindUidPids ( $iUid,
                                         \@hProcListHash,
                                         \@hUidProcListHash );

  &logTrace(20048,"<%s> : Number of processes for User %s = %s",
                           $sMyName, $sUser, $iMatchedPids);

  #----------------------------------------
  # Confirm that processes were found.
  #----------------------------------------
  if(0 == $iProcessCount)
  {
    #--------------------------------------------------------------------------
    # Log error and return unknown - if we cant get a process list, we dont
    # know the state.
    #--------------------------------------------------------------------------
    &logError(20049,
    "<%s> Call to get process list did not return any processes", $sMyName);
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  else
  {
    &logTrace( 20050,
    "<%s> Call to get process list returned [%s] processes",
    $sMyName, $iProcessCount);

    #----------------------------------------
    # Test for the StopProgram process.
    #----------------------------------------

    my @hProcListHashMatched=();
    my @hGZProcListHashMatched=();
    my @hNGZProcListHash=();
    my $iExpectedProcs = 0;

    #-----------------------------------------------------------------------------
    # Get the process corresponding to the Component
    #-----------------------------------------------------------------------------

    my @lsComponentProcess = OracleAS9::GetComponentProcess($sComponentProcessType,
                                                            $sOracleHome,
                                                            $iExpectedProcs,
                                                            $sOracleInstance,
                                                            $sOracleInstanceHome);

    my @lsOsDetails = POSIX::uname();
    my $iRel = $lsOsDetails[2];
    &logTrace(20051,"<%s>:Operating System revision is [%s].",$sMyName, $iRel);

    #------------------------------------------------
    # Check if Agent is running in Solaris Local Zone
    #------------------------------------------------

    if ( ( Arch::SUN_OS == $sOSType ) && ( ( $iRel ne "5.8" ) && ( $iRel ne "5.9" ) ) )
    {
       #--------------------------------------------
       # Find out the Zone Scheduler for this zone
       #--------------------------------------------
       foreach my $TmpProc (@hProcListHash)
       {
          if ( defined $TmpProc->{"PID"} && defined $TmpProc->{"PPID"})
          {
             $iZsched1 = $TmpProc->{'PID'};
             $iZsched2 = $TmpProc->{"PPID"};

             if( $iZsched1 == $iZsched2 )
             {
               if( 0 != $iZsched1 )
               {
                  &logTrace(20052,"<%s>:Zone Scheduler PID is [%s]",$sMyName, $iZsched1);
                  $bZoneFlag = 1;
                  last;
               }
             }
           }   
        }
    }

    #--------------------------------------
    # Find All the Procs for the Component
    #--------------------------------------

    my $iProcsFound = 0;
    foreach my $sComponentProcess ( @lsComponentProcess ) 
    { 
       @hProcListHashMatched = ();
       my $sFilter = ();
       $sFilter = "^" . $sOracleHome . "/$sComponentProcess";
  
       my $j2eeDirName = $sOracleHome . "/j2ee/" . $sComponentProcessType ;
       if ( -e  $j2eeDirName )
       {
          $sFilter = "^" . $sOracleHome . "/$sComponentProcess" . ".*" . "-Doracle.oc4j.instancename=$sComponentProcessType " . ".*";
       }
  
       if ( $sComponentProcessType =~ m/dcm-daemon/ )
       {
          $sFilter = "^" . $sOracleHome . "/$sComponentProcess" . ".*" . "dcm.jar" . ".*";
       }

       if ( $sComponentProcessType =~ m/OID/ || 
            $sComponentProcessType =~ m/OVD/ || 
            $sComponentProcessType =~ m/ReportsServerComponent/ || 
            $sComponentProcessType =~ m/iasconsole/ || 
            $sComponentProcessType =~ m/dbconsole/ ||
            $sComponentProcessType =~ m/agent/ ||
            $sComponentProcessType =~ m/performance_server/ || 
            $sComponentProcessType =~ m/messaging_server/ )
       {
              $sFilter = "^" . "$sComponentProcess" . ".*";
       }

       my $iNumberOfMatches = Proc::FilterProcs($sFilter,
                                                \@hUidProcListHash,
                                                \@hProcListHashMatched);

       &logTrace( 20053, "<%s> Matches for process with filter [%s] got [%d] matches",
                          $sMyName, $sFilter, $iNumberOfMatches);

       if ( 0 == $iNumberOfMatches  )
       {
         #---------------------------------------------
         # Offline at this point.  No procs to look at.
         #---------------------------------------------
         $iReturnCode = VCSDef_MonitorExit::OFFLINE;
         $iProcsFound = 0;
       }
       else
       {
         if ( ( Arch::SUN_OS == $sOSType ) && ( ( $iRel ne "5.8" ) && ( $iRel ne "5.9" ) ) )
         {
             if ( 1 == $bZoneFlag ) # local zone
             {
                &logTrace(20054,"<%s> : Number of Matched local zone process [%s].",$sMyName, $iNumberOfMatches);
             }
             else  # Gobal zone
             {
                #--------------------------------
                # Get count of Global Zone procs
                #--------------------------------
                my @lsPids = map { $_ -> { PID } } @hProcListHashMatched;
                my $sPids = join (',',@lsPids);

                @hGZProcListHashMatched = ();
                @hNGZProcListHash = ();

                OracleAS9::GetGZPidDetails($sPids,\@hGZProcListHashMatched,\@hNGZProcListHash);
                $iNumberOfMatches = scalar(@hGZProcListHashMatched);
 
                @lsPids = map { $_ -> { PID } } @hGZProcListHashMatched;
                &logTrace(20055,"<%s>:Got [%d] Matched global zone process [%s].", $sMyName, $iNumberOfMatches, join (',',@lsPids) );
                @hProcListHashMatched = @hGZProcListHashMatched;

                if ( 0 < scalar(@hNGZProcListHash )) { 
                  @lsPids = map { $_ -> { PID } } @hNGZProcListHash;
                  &logTrace(20056,"<%s>:Skipping local zone process [%s]",$sMyName, join (',',@lsPids) );
                }  
             }#End else
          }#End if

          if ( 0 < $iNumberOfMatches )
          {
             $iProcsFound = $iProcsFound + $iNumberOfMatches;
             push (@$rhlProcsMatched, @hProcListHashMatched); 
             &logTrace(20057,"<%s> : Total <%d> matches found .",$sMyName, $iProcsFound );
          }
       }#End else
    }#End foreach
  
    if ( 0 == $iProcsFound ) 
    {
       &logTrace(20058,"<%s> : Did not find any processes for [%s]", $sMyName, $sComponentProcessType );
       $iReturnCode = VCSDef_MonitorExit::OFFLINE;
    }
    elsif ( $iProcsFound < $iExpectedProcs )
    {      
       &logTrace(20059,"<%s> : Did not find all processes for [%s], Expected Procs [%d]", $sMyName, $sComponentProcessType, $iExpectedProcs );
       $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;
    } 
    else 
    { 
       my @lsPids = map { $_ -> { PID } } @$rhlProcsMatched;
       &logTrace(20060,"<%s> : Found all [%d] processes for [%s] : [%s], Expeced procs [%d]", 
                              $sMyName, $iProcsFound, $sComponentProcessType,join (',',@lsPids), $iExpectedProcs );
       $iReturnCode = VCSDef_MonitorExit::ONLINE;
    }

  }#End else
  return $iReturnCode;
}

=pod

#------------------------------------------------------------------------------
 Function : int SecondLevelMonitor ( IN String  OracleHome,
                                     IN String  User,
                                     IN String  ComponentProcessType,
                                     IN Integer TimeOut );

 Description : Run the SecondLevel Monitor which will check for a status
               of the component as seen by OPMN

 Parameters :

 $sOracleHome              ORACLE_HOME value for Oracle Application Server
 $sUser                    UNIX user for Oracle Application Server
 $sComponentProcessType    Component Name of the specific OracleAS component
 $iTimeOut                 Timeout value to run the status command

 Returns : Integer value as defined below:
           0  No matching processes found or error getting the processes
           1  Matching processes found or error getting the processes

#-------------------------------------------------------------------------------

=cut

sub SecondLevelMonitor()
{
  my ($sOracleHome,
      $sUser,
      $sComponentProcessType,
      $iTimeOut,$sOracleInstance, $sOracleInstanceHome) = @_ ;

  my ($sMyName ) = (caller 0)[3];

  &logTrace( 20061, "Subroutine: <%s> called with
             OracleHome     [%s]
             User           [%s]
             ComponentProcessType  [%s]
             TimeOut     [%s]
             OracleInstanceName [%s]
             OracleInstanceHome [%s]",
             $sMyName, $sOracleHome , $sUser , $sComponentProcessType , $iTimeOut, 
             $sOracleInstance, $sOracleInstanceHome);


  my $iExitCode = 0;
  my @lsCommandOut = ();
  my $bTimedOut;

  my $sServerStatusCmd = "$sOracleHome/opmn/bin/opmnctl";
  my $sServerStatusArgs = "status  -noheaders -fmt prt%sta ";

  if ($sOracleInstance =~ /\S+/ && $sOracleInstanceHome =~ /\S+/){
      $sServerStatusCmd = $sOracleInstanceHome."/"."bin/opmnctl";
  }


  #-----------------------------------------------------------------------------
  # If component is opmn itself, we just run status and check the exit code
  #-----------------------------------------------------------------------------
  if ( $sComponentProcessType eq "OPMN" )
  {
    $sServerStatusArgs = "status";
  }

  if ( $sComponentProcessType eq "iasconsole" ||
       $sComponentProcessType eq "dbconsole"  ||
       $sComponentProcessType eq "agent" )
  {
    $sServerStatusCmd = "$sOracleHome/bin/emctl";
    if ( $sOracleInstance =~ /\S+/ && $sOracleInstanceHome =~ /\S+/ ){
         $sServerStatusCmd = $sOracleInstanceHome."/EMAGENT/emagent_".$sOracleInstance."/bin/emctl";
    }			 
    $sServerStatusArgs = "status $sComponentProcessType";
  }

  my $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut("",
                                                     "",
                                                     $sServerStatusCmd,
                                                     $sServerStatusArgs,
                                                     \$iExitCode,
                                                     $sUser,
                                                     \@lsCommandOut,
                                                     \$bTimedOut,
                                                     $iTimeOut,
                                                     "",
                                                     "$sOracleHome/opmn/bin");

  if ( $iReturnCode )
  {
    #---------------------------------------------------------------------------
    # Command got executed 
    #---------------------------------------------------------------------------
    if ( $iExitCode )
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20062,"<%s>:[%s] [%s] failed with exit code [%s]",
                     $sMyName,$sServerStatusCmd,$sServerStatusArgs,$iExitCode);

      if ( $sComponentProcessType eq "OPMN"       || 
           $sComponentProcessType eq "iasconsole" ||
           $sComponentProcessType eq "dbconsole"  ||
           $sComponentProcessType eq "agent"  )
      {
        $iReturnCode = VCSDef_MonitorExit::OFFLINE;
      }
      else
      {
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
    }
    else 
    {
      &logTrace (20063,"<%s>:[%s] [%s] executed successfully ",
                     $sMyName,$sServerStatusCmd,$sServerStatusArgs);

      if ( $sComponentProcessType eq "OPMN"       ||
           $sComponentProcessType eq "iasconsole" ||
           $sComponentProcessType eq "dbconsole"  || 
           $sComponentProcessType eq "agent" )
      {
          $iReturnCode = VCSDef_MonitorExit::ONLINE;
      }
      else  ## OPMN Managed Components
      {
          SLMPARSE : foreach my $sOutputLine ( @lsCommandOut )
          {
              $sOutputLine =~ s/ //g;
              last SLMPARSE if ( $sOutputLine eq "\n" );
              (my $sComp,my $sState) = split ( /\|/, $sOutputLine );

              #-----------------------------------------------
              # For Oracle Application Server 10.1.3 SOA Suite
              #-----------------------------------------------

              (my $sCompFirst ,my $sComponentName) = split ( /\:/, $sComp );
              if ( $sCompFirst eq "OC4J" ){
                   $sComp = $sComponentName;
              } elsif ( $sOracleInstance =~ /\S+/ && $sComponentProcessType eq "WebCacheAdmin" ){
                   # The opmnctl status displays "WebCacheAdmin" component process type as "WebCache-admin" for Oracle Fusion
                   # Middleware installations
                   $sComponentProcessType = "WebCache-admin";
              } elsif ( $sOracleInstance =~ /\S+/ && $sComponentProcessType eq "OID" ){
                   # The opmnctl status displays "OID" component process type as "oidmon" for Oracle Fusion
                   # Middleware installations
                   $sComponentProcessType = "oidmon";
              }

              if ( $sComp eq $sComponentProcessType )
              {
                 chomp ( $sState );
                 &logTrace (20064,"<%s>:The State of [%s] is [%s]",
                                 $sMyName,$sComp,$sState);

                 if ($sState eq "Alive" )
                 {    
                     &logTrace (20065,"<%s>:Resource is ONLINE",$sMyName);
                     $iReturnCode = VCSDef_MonitorExit::ONLINE;
                 }
                 else
                 {
                     &logTrace (20066,"<%s>:Resource is OFFLINE",$sMyName);
                     $iReturnCode = VCSDef_MonitorExit::OFFLINE;
                 }
                 
              } 
           } # End SLMPARSE
       } # End else
    } # End else
  } # End if
  else
  {
    #-----------------------------------------------------------------------------
    # Command timed out
    #-----------------------------------------------------------------------------

    if ( $bTimedOut )
    {
      &logError (20067,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sServerStatusCmd,$iTimeOut);
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20068,"<%s>:[%s] failed.", $sMyName,$sServerStatusCmd);
    }
    $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
  }
  return $iReturnCode;
}

=pod

#------------------------------------------------------------------------------
 Function : String GetInstallType ( IN String  OracleHome);

 Description : 
   Get the Install Type ( for Information purpose only ) 

 Parameters :
   $sOracleHome       ORACLE_HOME value for Oracle Application Server

 Returns : 
   The value of the InstallType ( Infrastructure, ApplicationServer)

#-------------------------------------------------------------------------------

=cut

sub GetInstallType
{

  my ( $sOracleHome ) = @_; 

  my $sMyName = ( caller 0 )[3];

  my $myPropertiesFile = "$sOracleHome/config/ias.properties";
  my $iReturn = open(FH,"$myPropertiesFile");

  if ( ! $iReturn )
  {
      &logTrace(20069, "<%s>: Cannot open properties file %s", $sMyName, $myPropertiesFile);
  }
  else
  {
  my @FA = <FH> ;

  my $line;
  my $lval;
  my $rval;

  foreach $line ( @FA )
  {
     ($lval,$rval) = split ( /=/, $line );
     if ( $lval =~ /InstallType/ )
     {
         return ( $rval);
     }
  }
  &logTrace(20070, "<%s>:Did not find Installtype in ias.properties",$sMyName);
  }
}
 
=pod 

#-------------------------------------------------------------------------------
 Function    :  GetComponentProcess ( IN String ComponentProcessType,
                                      IN String OracleHome 
                                      OUT Integer ExpectedProcs);

 Description :  Get the process name for a given component

 Parameters  :

 $sComponentProcessType    Component Name of the specific OracleAS component
 $sOracleHome              Oracle Home Dir path
 $iExpectedProcs           Number of Proces Expected to run

 Returns     : List of Procs Filters to be Matched 

#-------------------------------------------------------------------------------

=cut 

sub GetComponentProcess()
{
   my ( $sComponentProcessType , $sOracleHome, $iExpectedProcs, $sOracleInstance, $sOracleInstanceHome ) = @_; 

   my $sMyName = ( caller 0 )[3];

   &logTrace ( 20071, "<%s>: Check proc for component [%s], Oracle Home [%s] , Oracle Instance [%s] & Oracle InstanceHome [%s]", 
                       $sMyName, $sComponentProcessType, $sOracleHome, $sOracleInstance, $sOracleInstanceHome);

   my @MONITOR_PROC_LIST = ();

   $iExpectedProcs = 0;
   my $j2eeDirName = $sOracleHome."/j2ee/".$sComponentProcessType ;
   if ( -e  $j2eeDirName ) 
   {
      &logTrace ( 20072, "<%s>: Monitoring Component Process Type [%s] ", $sMyName, $sComponentProcessType);
      @MONITOR_PROC_LIST = ( ".*/java" );
   }
   else
   { 
      #-----------------------
      # If not J2EE Proc then 
      #-----------------------
   
      my $perlPath = "$sOracleHome/perl/bin/perl";
      my $iasProc2 = "$sOracleHome/.*/java" . ".*" . "oracle.oc4j.localhome=$sOracleHome/sysman" . ".*" . "iasconfig_loc";
      my $dbcProc2 = "$sOracleHome/.*/java" . ".*" . "OC4J_DBConsole_";

      my $agentProc2 = "$sOracleHome/bin/emagent";
      my @OPMNPROC_LIST                 = ( "opmn/bin/opmn" );
      my @HTTPSERVERPROC_LIST           = ( "Apache/Apache/bin/httpd", "Apache/Apache/bin/rotatelogs" );
      my @OIDPROC_LIST                  = ( "$sOracleHome/bin/oidmon" , "$sOracleHome/bin/oidldap" , "oidldap" );
      my @WEBCACHEPROC_LIST             = ( "webcache/bin/webcached -OPMN" );
      my @WEBCACHEADMINPROC_LIST        = ( "webcache/bin/webcached -A -OPMN" );
      my @OVDPROC_LIST                  = ();
      my @REPORTSSERVERPROC_LIST         = ();

      if (($sOracleInstance =~ /\S+/) && ($sOracleInstanceHome =~ /\S+/)){
          $agentProc2                   = "$sOracleHome/bin/emagent".".*"."EMHOME=\\S+\/emagent_$sOracleInstance";
          @OIDPROC_LIST                 = ( "$sOracleHome/bin/oidmon.*INSTANCE_NAME=$sOracleInstance" , "$sOracleHome/bin/oidldap.*INSTANCE_NAME=$sOracleInstance" , "oidldap" );
          @WEBCACHEPROC_LIST            = ( "webcache/bin/webcached -OPMN.*INSTANCE_NAME=$sOracleInstance" );
          @WEBCACHEADMINPROC_LIST       = ( "webcache/bin/webcachea -OPMN.*-A.*INSTANCE_NAME=$sOracleInstance" );
          @OVDPROC_LIST                 = ( "$sOracleHome/.*/java".".*"."-Doracle.component.type=OVD".".*"."INSTANCE_NAME=$sOracleInstance");
          @REPORTSSERVERPROC_LIST       = ( "$sOracleHome/bin/rwserver.*INSTANCE_NAME=$sOracleInstance" , "/bin/sh.*$sOracleInstanceHome/config/reports/bin/rwserver.sh" );
      }

      my @IASCONSOLEPROC_LIST           = ( "$perlPath $sOracleHome/bin/emwd.pl iasconsole", $iasProc2 );
      my @DBCONSOLEPROC_LIST            = ( "$perlPath $sOracleHome/bin/emwd.pl dbconsole", $dbcProc2 );
      my @EMAGENTPROC_LIST              = ( "$perlPath $sOracleHome/bin/emwd.pl agent", $agentProc2 );
      my @DCMPROC_LIST                  = ( ".*/java" );

      my @SRVCSTATPROC_LIST             = ( "bin/DiscSrvcStat" );
      my @PRSRVRPROC_LIST               = ( "bin/dis51pr" );
      my @SESSRVRPROC_LIST              = ( "bin/dis51ws" );
      my @WLMSGSRVRPROC_LIST            = ( ".*/java.*-Doracle.ons.oraclehome=$sOracleHome.*messaging" );
      my @WLPERFSRVRPROC_LIST           = ( ".*/java.*-Doracle.ons.oraclehome=$sOracleHome.*performance" );

      my @OHSPROC_LIST                  = ( "ohs/bin/httpd" );

      SWITCH: {
         if ( $sComponentProcessType eq "OPMN" ) {
            @MONITOR_PROC_LIST = @OPMNPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "HTTP_Server" ) {
            @MONITOR_PROC_LIST = @HTTPSERVERPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "OID" ) {
            @MONITOR_PROC_LIST = @OIDPROC_LIST;
            $iExpectedProcs = 3;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "OVD" ) {
            @MONITOR_PROC_LIST = @OVDPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "ReportsServerComponent" ) {
            @MONITOR_PROC_LIST = @REPORTSSERVERPROC_LIST;
            $iExpectedProcs = 2;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "WebCache" ) {
            @MONITOR_PROC_LIST = @WEBCACHEPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "WebCacheAdmin" ) {
            @MONITOR_PROC_LIST = @WEBCACHEADMINPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "iasconsole" ) {
            @MONITOR_PROC_LIST = @IASCONSOLEPROC_LIST;
            $iExpectedProcs = 2;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "dbconsole" ) {
            @MONITOR_PROC_LIST = @DBCONSOLEPROC_LIST;
            $iExpectedProcs = 2;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "agent" ) {
            @MONITOR_PROC_LIST = @EMAGENTPROC_LIST;
            $iExpectedProcs = 2;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "dcm-daemon" ) {
            @MONITOR_PROC_LIST = @DCMPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "ServicesStatus" ) {
            @MONITOR_PROC_LIST = @SRVCSTATPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
         }
         if ( $sComponentProcessType eq "PreferenceServer" ) {
            @MONITOR_PROC_LIST = @PRSRVRPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
	     }
         if ( $sComponentProcessType eq "SessionServer" ) {
            @MONITOR_PROC_LIST = @SESSRVRPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
	     }
         if ( $sComponentProcessType eq "performance_server" ) {
            @MONITOR_PROC_LIST = @WLPERFSRVRPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
	     }
         if ( $sComponentProcessType eq "messaging_server" ) {
            @MONITOR_PROC_LIST = @WLMSGSRVRPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
	     }
         if ( $sComponentProcessType eq "OHS" ) {
            @MONITOR_PROC_LIST = @OHSPROC_LIST;
            $iExpectedProcs = 1;
            last SWITCH;
	     }
      }# End SWITCH
   }# End Else
   $_[2] = $iExpectedProcs;
   return @MONITOR_PROC_LIST;
}
 
=pod

 -------------------------------------------------------------------------
 Function : bool GetGZPidDetails ( IN String Pids,
                                 OUT RefList GzProcList, 
                                 OUT RefList NgzProcList );

 Description : Gets Global zone process details for a comma 
               seperated Pid list 

 Parameters :

 $sPids          Comma seperated Pids whose details are requested
 $rlGzProcList   Ref to List of Global Zone Procs
 $rlNgzProcList  Ref to List of Non Global Zone Procs

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub GetGZPidDetails 
{
  my ( $sPids,
       $rlGzProcList, 
       $rlNgzProcList ) = @_;
    
  #---------------
  # Anounce Entry
  #---------------
  my $sMyName = ( caller 0 )[3];
 
  &logTrace( 20081, "Subroutine <%s> called with ProcIds  [%s]",
             $sMyName, $sPids );

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #---------------
  # Validate Pids
  #---------------
  unless ( defined $sPids  ) { 
    &logError ( 20082,
    "<%s> Pids list is invalid", $sMyName );
    return 0;
  }

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sEnv, $sPsCmd, $sPsCmdArgs, $sLang, $sFmt, $iExitCode ) = ();
  my ( $iUID, $iPID, $iPPID, $sZONE, @lsArr ) = ();

  @$rlGzProcList = (); # Truncate..
  @$rlNgzProcList = (); # Truncate..

  #-------------
  # Store LANG..
  #-------------
  $sLang = $ENV { LANG };
  $ENV { LANG } = 'C';

  #-------------
  # Set Command
  #-------------

  $sPsCmd = "/bin/ps";
  $sPsCmdArgs = "-Z -o uid,pid,ppid,zone -p $sPids";

  $sFmt = '([\d]+)\s+([\d]+)\s+([\d]+)\s+(.*)';

  #-------------------------------
  # Fire command and get details..
  #-------------------------------
  @lsArr = 
  map {
    s/\n//;                             # Chomp..
    s/^\s+//;                           # Strip leading spaces..
    /^$sFmt$/o;                         # Extract fields..
  } readpipe ( "$sPsCmd $sPsCmdArgs" ); # From the command..

  $iExitCode = $? >> 8; # Get exit code..

  #------------------------
  # Restore env variables..
  #------------------------
  defined $sLang ? $ENV { LANG } = $sLang:delete $ENV { LANG };
 
  #------------------
  # Check exit code..
  #------------------
  if ( $iExitCode ) {
    &logTrace ( 20083,
    "<%s> Cmd [%s] with args [%s] returned with exit code [%s]",
    $sMyName, $sPsCmd, $sPsCmdArgs, $iExitCode );
    return 0;
  }

  #---------------------
  # Fill up the bucket..
  #---------------------

  while ( ( $iUID, $iPID, $iPPID, $sZONE ) = splice ( @lsArr, 0, 4 ) ) 
  {
     if ( $sZONE =~ m/global/ ) 
     {
       my %hsProc = ();
       @hsProc { qw ( UID PID PPID ZONE) } = ( $iUID, $iPID, $iPPID, $sZONE );
       push @$rlGzProcList, \%hsProc;
     }
     else  
     {
       my %hsProc = ();
       @hsProc { qw ( UID PID PPID ZONE) } = ( $iUID, $iPID, $iPPID, $sZONE );
       push @$rlNgzProcList, \%hsProc;
     }
  }
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int DelayEntryPoint ( IN Integer DelayPeriod,
                                  IN String  User,
                                  IN String  OracleHome,
                                  IN String  ComponentProcessType );

 In offline entry point this subroutine delays the entry point till the
 OracleAS server process stops running.

 Parameters :
  $iDelayPeriod           Time till the entry point could be delayed
  $sUser                  Unix User
  $sOracleHome            OracleHome Directory Path
  $sComponentProcessType  Component ProcessType to Monitor

 Returns:

  0 Failure         Failure in delaying the entry point.
  1 Success         Successfully delayed the entry point.

 -------------------------------------------------------------------------

=cut

sub DelayEntryPoint
{
  my ( $iDelayPeriod,
       $sUser,
       $sOracleHome,
       $sOracleInstance,
       $sOracleInstanceHome,
       $sComponentProcessType ) = @_;
   
  my $sMyName         = ( caller 0)[3];

  &logTrace ( 20084, "Subroutine <%s> called with:
  DelayPeriod              [%s]
  User                     [%s]
  OracleHome               [%s]
  ComponentProcessType     [%s]", 
  $sMyName, $iDelayPeriod, $sUser, $sOracleHome, $sComponentProcessType );

  my $iReturnCode = 0;
  my $iNSR   = 0;
  my $iSleepInterval = 5;
  my @hRunningProcs = ();

  $iReturnCode = OracleAS9::GetProcState( $sUser ,
                                          $sOracleHome,
                                          $sOracleInstance,
                                          $sOracleInstanceHome,
                                          $sComponentProcessType,
                                          \@hRunningProcs );

  if ( VCSDef_MonitorExit::OFFLINE == $iReturnCode )
  {
     $iReturnCode = 1;
  } 
  else
  { 
     $iNSR = $iDelayPeriod - time;
     while ( $iNSR > $iSleepInterval )
     {
       if ( $iNSR <= 5 )
       {
          if( $iNSR > 0 )
          {
             &logInfo(20085, "<%s>:Sleeping [%s] seconds", $sMyName, $iNSR );
             sleep $iNSR;
           }
          return 0;
        }
         
       $iReturnCode = OracleAS9::GetProcState( $sUser ,
                                               $sOracleHome,
                                               $sOracleInstance,
                                               $sOracleInstanceHome,
                                               $sComponentProcessType,
                                               \@hRunningProcs );

       if ( VCSDef_MonitorExit::OFFLINE != $iReturnCode )
       { 
          &logTrace(20086,"<%s>:Server Processes are running. ",$sMyName );
          my @lsPids = map { $_ -> { PID } } @hRunningProcs;
          &logInfo(20087, "<%s>:Sleeping for [%s] seconds as [%s] procs are running.", 
                          $sMyName, $iSleepInterval, join (',', @lsPids ) );
          sleep $iSleepInterval;
          $iNSR = $iDelayPeriod - time;
        }
       else
       {
          &logTrace(20088, "<%s>:Server processes not running.", $sMyName);
          $iReturnCode = 1;
          last;
       }
     } # End while
  }
  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int GetEngineTimeout ( IN Bool UseMargin );

 Description : Calculates engine timeout for entry point
 
 Parameters  :

 $bUseMargin      Flag to indicate if safety margin is requested

                  boolean value in the set of {0,1} where
                  0 No safety margin needed
                  1 Safety margin to be used

 Returns:  Time in secs.
 -------------------------------------------------------------------------

=cut

sub GetEngineTimeout {
  #----------------
  # Get Arguments..
  #----------------
  my ( $bUseMargin ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20145, "Subroutine <%s> called with:
  UseMargin  [%s]",
  $sMyName, $bUseMargin );

  #-------------------------
  # Define local variables..
  #-------------------------
  my ( $sAttribute, $sCommand, $sCommandArgs,
       $iEngineTimeout, $iExitCode ) = ();

  #----------------------------------
  # Grace margins from VCS timeouts..
  #----------------------------------
  my %hiGraceMargin = ( 'MonitorTimeout', 10,   # Seconds..
                        'OnlineTimeout',  10,
                        'OfflineTimeout', 10,
                        'CleanTimeout',   20 );

  #--------------------------------------------------
  # Attempt to get resource level overridden values..
  #--------------------------------------------------
  my $sResourceName = $ENV { VCS_LOG_RESOURCE_NAME };
  my ( $sEntryPointName, $sResourceType ) = reverse $0 =~ m@([^/]+)@g;
  ( $sAttribute = $sEntryPointName ) =~ s/(.*)/\u$1Timeout/;

  #-----------------------------------
  # Query if it has already been set..
  #-----------------------------------
  if ( defined $::giEPTimeout &&    # Defined..
       $::giEPTimeout =~ /^\d+/ ) { # to an integer..
    $iEngineTimeout = $::giEPTimeout;

    &logTrace ( 20146,
    "<%s> Using AEPTimeout [%s] as EP timeout value",
    $sMyName, $iEngineTimeout );
  }
  else {
    #-----------------------------------------
    # Form query string based on entry point..
    #-----------------------------------------
    $sCommand = $sCmdHares; # Get global..
    $sCommandArgs = "-value $sResourceName $sAttribute";
  
    #----------------------------------
    # Fire command and collect output..
    #----------------------------------
    ( $iEngineTimeout = readpipe ( "$sCommand $sCommandArgs 2>/dev/null" ) )
                      =~ s/\n//;
  
    #--------------------
    # Check exit status..
    #--------------------
    if ( $iExitCode = $? >> 8 ) {
      &logTrace ( 20147,
      "<%s> Command [%s] with Args [%s] failed with exitcode [%s].",
      $sMyName, $sCommand, $sCommandArgs, $iExitCode );
    }
    elsif ( ! $iEngineTimeout ) { # Undefined value..
      &logTrace ( 20148, "<%s> Could not get [%s] from engine",
      $sMyName, $sAttribute );
    }
    else { # Success..
      &logTrace ( 20149, "<%s> Got [%s] as [%s] sec from engine",
      $sMyName, $sAttribute, $iEngineTimeout );
    }
  
    #-----------------------------------------
    # Form query string based on entry point..
    #-----------------------------------------
    unless ( $iEngineTimeout ) {
      $sCommand = $sCmdHatype; # Get global..
      ( $sAttribute = $sEntryPointName ) =~ s/(.*)/\u$1Timeout/;
      $sResourceType = "OracleAS9" if $sResourceType !~ /\w/; # Ensure..
      $sCommandArgs = "-value $sResourceType $sAttribute";
    
      #----------------------------------
      # Fire command and collect output..
      #----------------------------------
      ( $iEngineTimeout = readpipe ( "$sCommand $sCommandArgs" ) ) =~ s/\n//;
    
      #--------------------
      # Check exit status..
      #--------------------
      if ( $iExitCode = $? >> 8 ) {
        &logTrace ( 20150,
        "<%s> Command [%s] with Args [%s] failed with exitcode [%s]",
        $sMyName, $sCommand, $sCommandArgs, $iExitCode );
        $iEngineTimeout = $bUseMargin ? $hiGraceMargin { $sAttribute } : 0;
      }
      elsif ( ! $iEngineTimeout ) { # Empty value..
        &logTrace ( 20151, "<%s> Could not get [%s] from engine",
        $sMyName, $sAttribute );
        $iEngineTimeout = $bUseMargin ? $hiGraceMargin { $sAttribute } : 0;
      }
      else { # Success..
        &logTrace ( 20152, "<%s> Got [%s] as [%s] sec from engine",
        $sMyName, $sAttribute, $iEngineTimeout );
      }
    }
    else {
        &logTrace ( 20153, "<%s> Got [%s] as [%s] sec from engine",
        $sMyName, $sAttribute, $iEngineTimeout );
    }
  }

  #----------------------------
  # Apply margin if requested.. 
  #----------------------------
  if ( $bUseMargin     &&                                    # Margin requested..
       $iEngineTimeout >= $hiGraceMargin { $sAttribute } ) { # And valid..
    $iEngineTimeout -= $hiGraceMargin { $sAttribute };       # Apply..

    &logTrace ( 20154,
    "<%s> Setting up [%s] sec margin. Returning [%s]",
    $sMyName, $hiGraceMargin { $sAttribute }, $iEngineTimeout );
  }

  return ( $iEngineTimeout ); # Secs..
}

=pod

 -------------------------------------------------------------------------
 Function : int RunSecondLevelMonitor( IN Integer SecondLevelMonitor )

 Description :  Determines whether now is the time to run the Second
 Level Monitoring checks for the OracleAS9 resource. This subroutine 
 also maintains the slm counter file at the location :
 /tmp/.VRTS<ResourceType>/<ResourceName>.slm
 The file is updated every monitor cycle if SecondLevelMonitor is value
 is greater than 1. 

 Parameters :

  $iSecondLevelMonitor    SecondLevelMonitor attribute value

 Returns : Integer values as defined below.

  0  Second Level Monitor should not be run.
  1  Second Level Monitor need to be run.

 -------------------------------------------------------------------------

=cut

sub RunSecondLevelMonitor 
{
  #----------------
  # Get Arguments..
  #----------------
  my ( $iSecondLevelMonitor) = @_;
       
  my $sResourceType   = $ENV{VCS_LOG_AGENT_NAME};
  my $sResourceName   = $ENV{'VCS_LOG_RESOURCE_NAME'};

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20223, "Subroutine <%s> called:
  SecondLevelMonitor        [%s]
  ResourceType              [%s]
  ResourceName              [%s]",
  $sMyName, $iSecondLevelMonitor, $sResourceType, $sResourceName );

  #---------------------------
  # Set Return Code to false..
  #---------------------------
  my $iReturnCode = 0; # Do not run SLM..

  #------------------------
  # Setup local variables..
  #------------------------
  my $sSLMDir  = "/tmp/.VRTS$sResourceType";
  my $sSLMFile = "$sSLMDir/$sResourceName.slm";
  my $iSLMCntr = undef;

  #--------------------------------------------
  # Validate SecondLevelMonitor trivial cases..
  #--------------------------------------------
  if ( 0 == $iSecondLevelMonitor ||
       1 == $iSecondLevelMonitor ) 
  {
    $iReturnCode = $iSecondLevelMonitor; # Assign..

    &logTrace (20224, "<%s>:Removing SLM file [%s] if it exists. Returning with [%s]", $sMyName, $sSLMFile, $iSecondLevelMonitor );

    unlink $sSLMFile if ( -e $sSLMFile ); # Remove SLM file..

    return $iReturnCode;
  }

  #----------------------------------
  # Create history folder if needed..
  #----------------------------------
  if ( -d $sSLMDir ) 
  {
    &logTrace ( 20225, "<%s>:SLM folder [%s] exists", $sMyName, $sSLMDir );
  }
  else 
  {
    mkdir ( $sSLMDir, 0777 ) ||
    &logWarn ( 20226, "<%s>:Failed to create SLM folder [%s]",
    $sMyName, $sSLMDir );
    chmod (0777,$sSLMDir );
    return 1; # Run SLM..
  }
  
  #--------------------------
  # Create SLM history file..
  #--------------------------
  if ( sysopen ( SLMFILE, $sSLMFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) {
    &logTrace ( 20227,
    "<%s>:Opened SLM file [%s] successfully for read/write",
    $sMyName, $sSLMFile );
  }
  else {
    &logWarn ( 20228,
    "<%s>:Failed to open SLM file [%s] for read/write",
    $sMyName, $sSLMFile );
    return 1; # Run SLM..
  }
  
  #---------------
  # Read in file..
  #---------------
  ( $iSLMCntr ) = grep { s/\n// } <SLMFILE>; 

  #-----------------
  # Corrupted file..
  #-----------------
  if ( ! defined $iSLMCntr || $iSLMCntr < 0 ) {

    $iSLMCntr ||= ''; # Avoid undefs..

    &logTrace ( 20229,
    "<%s>:SLM file [%s] contains [%s]. Re-setting to [%s]",
    $sMyName, $sSLMFile, $iSLMCntr, $iSecondLevelMonitor );
    $iSLMCntr = $iSecondLevelMonitor; # Set to default..
  }
  
  #-----------------------------------------
  # Update counter, if less than requested..
  #-----------------------------------------
  if ( $iSecondLevelMonitor > $iSLMCntr ) {
    ++ $iSLMCntr;
  }
  else {
    $iSLMCntr = 1;    # Now's the time..
    $iReturnCode = 1; # Run SLM..
  }
  
  #------------------
  # Update the file..
  #------------------
  seek ( SLMFILE, 0, 0 ); # Rewind before update..

  &logTrace ( 20230,
  "<%s>:Updating SLM file [%s] with [%s]. Return code is [%s]",
  $sMyName, $sSLMFile, $iSLMCntr, $iReturnCode );

  print SLMFILE "$iSLMCntr\n"; # Update..
  
  close ( SLMFILE ) ||
  &logWarn ( 20231,
  "<%s>:Close failed on SLM file [%s] after update",
  $sMyName, $sSLMFile, $! );

  return $iReturnCode;
}

sub GetComponantPids
{
  my ($sOracleHome,
      $sOracleInstance,
      $sOracleInstanceHome,
      $sUser,
      $sComponentProcessType,
      $iTimeOut,
      $rliPids ) = @_ ;

  my ($sMyName ) = (caller 0)[3];

  &logTrace( 20311, "Subroutine: <%s> called with
             OracleHome     [%s]
             User           [%s]
             ComponentProcessType  [%s]
             TimeOut     [%s]
             OracleInstanceName [%s]
             OracleInstanceHome [%s]",
             $sMyName, $sOracleHome , $sUser , $sComponentProcessType , $iTimeOut, 
             $sOracleInstance, $sOracleInstanceHome);

  my $iExitCode = 0;
  my @lsCommandOut = ();
  my $bTimedOut;

  my $sServerStatusCmd = "";
  if ( $sOracleInstance =~ /\S+/ && $sOracleInstanceHome =~ /\S+/ ){
       $sServerStatusCmd = "$sOracleInstanceHome/bin/opmnctl";
  } else {
       $sServerStatusCmd = "$sOracleHome/opmn/bin/opmnctl";
  }

  my $sServerStatusArgs = "status  -noheaders -fmt prt%pid%sta ";

  my $iReturnCode;
  #-----------------------------------------------------------------------------
  # If component is opmn itself, we just run status and check the exit code
  #-----------------------------------------------------------------------------
  if ( $sComponentProcessType eq "OPMN" || 
       $sComponentProcessType eq "iasconsole" ||
       $sComponentProcessType eq "dbconsole"  ||
       $sComponentProcessType eq "agent" )
  {
      my ( @liPids, @hRunningProcs ) = ();

      $iReturnCode = GetProcState( $sUser ,
                                   $sOracleHome,
                                   $sOracleInstance,
                                   $sOracleInstanceHome,
                                   $sComponentProcessType,
                                   \@hRunningProcs );

   	  @$rliPids = map { $_ -> { PID } } @hRunningProcs;
  }
  else {
      $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut("",
                                                     "",
                                                     $sServerStatusCmd,
                                                     $sServerStatusArgs,
                                                     \$iExitCode,
                                                     $sUser,
                                                     \@lsCommandOut,
                                                     \$bTimedOut,
                                                     $iTimeOut,
                                                     "",
                                                     "$sOracleHome/opmn/bin");

      if ( $iReturnCode )
      {
          SLMPARSE : foreach my $sOutputLine ( @lsCommandOut )
          {
              $sOutputLine =~ s/ //g;
              last SLMPARSE if ( $sOutputLine eq "\n" );
              my ($sComp, $iPid, $sState) = split ( /\|/, $sOutputLine );
    
              #-----------------------------------------------
              # For Oracle Application Server 10.1.3 SOA Suite
              #-----------------------------------------------
    
              (my $sCompFirst ,my $sComponentName) = split ( /\:/, $sComp );
              if ( $sCompFirst eq "OC4J" )
              {
                  $sComp = $sComponentName;
              }
              # The opmnctl status displays "OID" Component Process type as "oidmon" for Oracle Fusion Middleware installations
              if ( $sComp eq "oidmon" )
              {
                  $sComp = "OID";
              }
              # The opmnctl status displays "WebCacheAdmin" component process type as "WebCache-admin" for Oracle Fusion Middleware
              # installations
              if ( $sComp eq "WebCache-admin" )
              {
                  $sComp = "WebCacheAdmin";
              }
              if ( $sComp eq $sComponentProcessType )
              {
                  chomp ( $sState );
                  &logTrace (20312,"<%s>:The State of [%s] is [%s]",
                                   $sMyName,$sComp,$sState);
    
                  if ($sState eq "Alive" )
                  {    
                     &logTrace (20313,"<%s>:Resource is ONLINE",$sMyName);
			    	 push ( @$rliPids, $iPid );
                     $iReturnCode = VCSDef_MonitorExit::ONLINE;
                  }
				  else {
                     $iReturnCode = VCSDef_MonitorExit::OFFLINE;
				  }
 	           } 
   		   } # End SLMPARSE
  	   } # End if
	   else {
           $iReturnCode = VCSDef_MonitorExit::OFFLINE;
       }
   }
   return $iReturnCode;
}
1; # So PERL knows its included..
